/* aggregate.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __AGGREGATE_H
#define __AGGREGATE_H 1

#include <vector>

#include "source.h"
#include "scheduler.h"
#include "directory.h"
#include "extractor.h"
#include "packetwriter.h"
#include "flowwriter.h"
#include "statwriter.h"
#include "flow2writer.h"
#include "namewriter.h"
#include "param.h"

namespace trafficmeter
{
    /**********************************************************************************************************/
    struct Aggregate
    {
        std::vector<Source*>&           sources;
        std::vector<Scheduler*>         schedulers;
        std::vector<Extractor*>         extractors;
        std::vector<PacketWriter*>      packetWriters;
        std::vector<FlowWriter*>        flowWriters;
        std::vector<StatWriter*>        statWriters;
        Flow2Writer*                    flow2Writer;
        NameWriter*                     nameWriter;
        Directory                       packetDirectory;
        Directory                       flowDirectory;
        Directory                       statDirectory;
        Directory                       flow2Directory;

                                        Aggregate( std::vector<Source*>&, const Param& );
                                        ~Aggregate();
        void                            start();
        void                            stop();
    };
}

#endif  // __AGGREGATE_H
