/* directory.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __DIRECTORY_H
#define __DIRECTORY_H 1

#include <atomic>
#include <ctime>
#include <map>
#include <string>
#include <vector>
#include <libiclass/common>
#include <libiclass/task>

#include "fileid.h"

namespace trafficmeter
{
    /**********************************************************************************************************/
    class Directory : public iclass::commont
    {
    public:
        class File;

                                        Directory( const std::string& dirName, int hashLength );
                                        ~Directory();

        void                            load();
        void                            loadCancel();
        void                            setQuota( int quota );
        File                            writingFile( int hash, time_t stem, int extension=0 );
        std::vector<File>               deletingFiles();
        bool                            isCreated() const;

    protected:
        struct Loader_;

        const Path                      path_;
        const int                       hashLength_;
        bool                            isCreated_;
        time_t                          createBound_;
        Loader_*                        loader_;
        std::atomic<int>                quota_a_;
        std::map<FileId,uintmax_t>      fileSizes_c_;
        uintmax_t                       totalBytes_c_;
    };

    /**********************************************************************************************************/
    class Directory::File : public FileId
    {
    public:
                                        File( Directory* dir, FileId id );
                                        File( Directory* dir, int hash, time_t stem, int extension=0 );

        void                            removeSize();
        void                            updateSize( uintmax_t );
        Path                            path() const;

    protected:
        Directory*                      dir_;
    };

    /**********************************************************************************************************/
    struct Directory::Loader_ : public iclass::taskt
    {
        Directory*                      dir_;

        virtual void                    routine() override;
    };
}

#endif  // __DIRECTORY_H
