/* extractor.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __EXTRACTOR_H
#define __EXTRACTOR_H 1

#include <vector>
#include <libiclass/task>

#include "packetwriter.h"
#include "flowwriter.h"
#include "flow2writer.h"
#include "namewriter.h"
#include "source.h"
#include "substratequeue.h"

namespace trafficmeter
{
    /**********************************************************************************************************/
    class Extractor : public iclass::taskt
    {
    public:
                                        Extractor( Source* source, int nChunks );
        virtual                         ~Extractor();

        void                            attach( PacketWriter* );
        void                            attach( FlowWriter* );
        void                            attach( Flow2Writer* );
        void                            attach( NameWriter* );
        SubstrateQueue*                 inputQueue();
        long                            queueSize() const;
        uintmax_t                       nProcessed() const;

    protected:
        virtual void                    routine() override;
        void                            extract_( Substrate* );

        Source*                         source_;
        SubstrateQueue                  inputQueue_;
        PacketCollection*               packetCollection_;
        std::vector<FlowCollection*>    flowCollections_;
        Flow2Collection*                flow2Collection_;
        NameCollection*                 nameCollection_;
        std::atomic<uintmax_t>          nProcessed_a_;
    };
}

#endif  // __EXTRACTOR_H
