/* flowcollection.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __FLOWCOLLECTION_H
#define __FLOWCOLLECTION_H 1

#include <map>
#include <vector>
#include <libiclass/common>

#include "flowrecord.h"
#include "metadata.h"
#include "slotindex.h"
#include "summary.h"

namespace trafficmeter
{
    /**********************************************************************************************************/
    class FlowCollection : public iclass::commont
    {
    public:
        using SlotVector                = std::vector<std::map<BeFlow,Summary>>;

                                        FlowCollection( int threadNo, int nThreads );
                                        ~FlowCollection();

        void                            push( const SlotIndex& index, const Metadata& meta );
        SlotVector*                     pull();
        time_t                          oldestGroup() const;
        bool                            isBusy() const;
        long                            nRows() const;

    protected:
        const int                       threadNo_;
        const int                       nSlots_;
        SlotVector*                     slots_c_;
        SlotVector*                     nextSlots_c_;
        time_t                          oldestGroup_c_;
        time_t                          newestGroup_c_;
        volatile mutable bool           hasActivity_c_;
    };
}

#endif  // __FLOWCOLLECTION_H
