/* flowwriter.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __FLOWWRITER_H
#define __FLOWWRITER_H 1

#include <vector>
#include <libiclass/task>

#include "directory.h"
#include "fileheader.h"
#include "flowcollection.h"
#include "statwriter.h"

namespace trafficmeter
{
    /**********************************************************************************************************/
    class FlowWriter : public iclass::taskt
    {
    public:
                                        FlowWriter( Directory* directory, int threadNo, int nThreads );
        virtual                         ~FlowWriter();

        void                            attach( StatWriter* );
        FlowCollection*                 createCollection();
        long                            nRows() const;
        uintmax_t                       nWritten() const;

    protected:
        virtual void                    routine() override;
        void                            wait_() const;
        void                            pull_();
        bool                            fwrite_();

        const int                       threadNo_;
        const int                       nThreads_;
        const int                       nSlots_;
        Directory*                      directory_;
        std::vector<StatCollection*>    statCollections_;
        std::vector<FlowCollection*>    collections_;
        FlowCollection::SlotVector*     combineSlots_;
        FileHeader**                    headers_;
        time_t                          group_;
        std::atomic<uintmax_t>          nWritten_a_;
    };
}

#endif  // __FLOWWRITER_H
