/* groupbytime.cc
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include <libiclass/period>

#include "groupbytime.h"
#include "tmdefs.h"

using namespace iclass;
using namespace trafficmeter;

/**************************************************************************************************************/
GroupByTime::GroupByTime() :
    beginCache_( 0 ),
    endCache_( 0 )
{
}

/**************************************************************************************************************/
GroupByTime::~GroupByTime()
{
}

/**************************************************************************************************************/
time_t
GroupByHour::operator()( time_t time )
{
    if ( time < beginCache_ || time >= endCache_ )
    {
        beginCache_ = moduloRoundDown( time, 3600 );
        endCache_ = beginCache_ + 3600;
    }

    return beginCache_;
}

/**************************************************************************************************************/
time_t
GroupByDay::operator()( time_t time )
{
    if ( time < beginCache_ || time >= endCache_ )
    {
        periodt  period( time, periodt::PLDAY );

        beginCache_ = period.begin().unixtime();
        endCache_ = period.end().unixtime();
    }

    return beginCache_;
}

/**************************************************************************************************************/
time_t
GroupByMonth::operator()( time_t time )
{
    if ( time < beginCache_ || time >= endCache_ )
    {
        periodt  period( time, periodt::PLMONTH );

        beginCache_ = period.begin().unixtime();
        endCache_ = period.end().unixtime();
    }

    return beginCache_;
}

/**************************************************************************************************************/
time_t
IgnoreTime::operator()( time_t time )
{
    return 0;
}
