/* macaddress.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2021 Askar Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __MACADDRESS_H
#define __MACADDRESS_H 1

#include <cstdint>
#include <cstring>
#include <string>

namespace trafficmeter
{
    /**********************************************************************************************************/
    class MacAddress
    {
    public:
                                        MacAddress();
                                        MacAddress( const uint8_t* );
        explicit                        MacAddress( std::string );

        bool                            isBroadcast() const;
        std::string                     str() const;

        friend bool                     operator==( const MacAddress&, const MacAddress& );
        friend bool                     operator!=( const MacAddress&, const MacAddress& );
        friend bool                     operator<( const MacAddress&, const MacAddress& );
        friend bool                     operator>( const MacAddress&, const MacAddress& );
        friend bool                     operator<=( const MacAddress&, const MacAddress& );
        friend bool                     operator>=( const MacAddress&, const MacAddress& );

    protected:
        uint8_t                         address_[ 6 ];
    };
}

/*
****************************************************************************************************************
****************************************************************************************************************
*/

namespace trafficmeter
{
    //----------------------------------------------------------------------------------------------------------
    inline
    MacAddress::MacAddress()
    {
        memset( address_, 0, 6 );
    }

    //----------------------------------------------------------------------------------------------------------
    inline
    MacAddress::MacAddress( const uint8_t* mac )
    {
        memcpy( address_, mac, 6 );
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    MacAddress::isBroadcast() const
    {
        return address_[ 0 ] & 1;
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    operator==( const MacAddress& lhs, const MacAddress& rhs )
    {
        return memcmp( lhs.address_, rhs.address_, 6 ) == 0;
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    operator!=( const MacAddress& lhs, const MacAddress& rhs )
    {
        return memcmp( lhs.address_, rhs.address_, 6 ) != 0;
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    operator<( const MacAddress& lhs, const MacAddress& rhs )
    {
        return memcmp( lhs.address_, rhs.address_, 6 ) < 0;
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    operator>( const MacAddress& lhs, const MacAddress& rhs )
    {
        return memcmp( lhs.address_, rhs.address_, 6 ) > 0;
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    operator<=( const MacAddress& lhs, const MacAddress& rhs )
    {
        return memcmp( lhs.address_, rhs.address_, 6 ) <= 0;
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    operator>=( const MacAddress& lhs, const MacAddress& rhs )
    {
        return memcmp( lhs.address_, rhs.address_, 6 ) >= 0;
    }
}

#endif  // __MACADDRESS_H
