/* monitor.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __MONITOR_H
#define __MONITOR_H 1

#include <vector>
#include <libiclass/task>

#include "param.h"
#include "sweeper.h"

namespace trafficmeter
{
    class Aggregate;

    /**********************************************************************************************************/
    class Monitor : public iclass::taskt
    {
    public:
                                        Monitor( const Param& param );
        virtual                         ~Monitor();

        void                            bind( Aggregate* ag );
        bool                            reconfigure( const Param& param );

    protected:
        virtual void                    routine() override;
        void                            checkDropped_();
        void                            printState_();
        void                            printSweep_();

        std::vector<Aggregate*>         aggregates_;
        Sweeper                         sweeper_;
        int                             verbosity_;
        int                             flowQuota_;
        int                             packetQuota_;
        int                             statQuota_;
        int                             flow2Quota_;
        uintmax_t                       nReceivedPrevious_;
        uintmax_t                       nDroppedPrevious_;
        uintmax_t                       nPacketsWrittenPrevious_;
        uintmax_t                       nFlowsWrittenPrevious_;
        uintmax_t                       nStatsWrittenPrevious_;
        uintmax_t                       nNamesWrittenPrevious_;
    };
}

#endif  // __MONITOR_H
