/* namecollection.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __NAMECOLLECTION_H
#define __NAMECOLLECTION_H 1

#include <cstring>
#include <map>
#include <string>
#include <libiclass/common>
#include <libiclass/ipv4>

#include "metadata.h"
#include "packet.h"
#include "tmdefs.h"

namespace trafficmeter
{
    /**********************************************************************************************************/
    class NameCollection : public iclass::commont
    {
    public:
        struct Key;

                                        NameCollection();
                                        ~NameCollection();

        void                            push( const Metadata&, Packet );
        std::map<Key,int>*              pull();
        long                            nRows() const;

    protected:
        iclass::iplookup4t*             exclusions_;
        std::map<Key,int>*              map_c_;
    };

    /**********************************************************************************************************/
    struct NameCollection::Key
    {
        iclass::ipaddr4t                ip;
        std::string                     host;

                                        Key( iclass::ipaddr4t, const std::string& );
        bool                            operator<( const Key& ) const;
    };
}

/*
****************************************************************************************************************
****************************************************************************************************************
*/

namespace trafficmeter
{
    //----------------------------------------------------------------------------------------------------------
    inline
    NameCollection::Key::Key( iclass::ipaddr4t ip, const std::string& host ) :
        ip( ip ),
        host( host )
    {
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    NameCollection::Key::operator<( const Key& rhs ) const
    {
        if ( ip != rhs.ip )
        {
            return ip < rhs.ip;
        }

        return host < rhs.host;
    }
}

#endif  // __NAMECOLLECTION_H
