/* packet.cc
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include <cassert>
#include <cstring>
#include <new>
#include <arpa/inet.h>

#include "packet.h"
#include "substrate.h"

using namespace trafficmeter;

/**************************************************************************************************************/
inline static Substrate*
substrate_( Packet::Body* body )
{
    return reinterpret_cast<Substrate*>( reinterpret_cast<uint8_t*>(body) + body->offsetSubstrate );
}

/**************************************************************************************************************/
Packet::Packet() :
    body_( nullptr )
{
}

/**************************************************************************************************************/
Packet::Packet( Body* packetBody ) :
    body_( packetBody )
{
    substrate_( body_ )->refIncrement();
}

/**************************************************************************************************************/
Packet::Packet( const Packet& packet )
{
    if ( packet.body_ != nullptr )
    {
        substrate_( packet.body_ )->refIncrement();
    }

    body_ = packet.body_;
}

/**************************************************************************************************************/
Packet::~Packet()
{
    if ( body_ != nullptr )
    {
        substrate_( body_ )->refDecrement();
    }
}

/**************************************************************************************************************/
Packet&
Packet::operator=( const Packet& packet )
{
    if ( packet.body_ != nullptr )
    {
        substrate_( packet.body_ )->refIncrement();
    }

    if ( body_ != nullptr )
    {
        substrate_( body_ )->refDecrement();
    }

    body_ = packet.body_;
    return *this;
}
