/* packet.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __PACKET_H
#define __PACKET_H 1

#include <cstdint>

#include "timestamp.h"
#include "tmdefs.h"

namespace trafficmeter
{
    /**********************************************************************************************************/
    class Packet
    {
    public:
        struct Body;

                                        Packet();
                                        Packet( Body* );
                                        Packet( const Packet& );
                                        ~Packet();

        Packet&                         operator=( const Packet& );
        Timestamp                       timestamp() const;
        const uint8_t*                  data2() const;
        unsigned                        nBytes2() const;

    protected:
        Body*                           body_;
    };

    /**********************************************************************************************************/
    struct Packet::Body
    {
        Timestamp                       timestamp;
        int32_t                         offsetSubstrate;
        int16_t                         offsetLayer2;
        uint16_t                        nBytes2;
    };
}

/*
****************************************************************************************************************
****************************************************************************************************************
*/

namespace trafficmeter
{
    //----------------------------------------------------------------------------------------------------------
    inline Timestamp
    Packet::timestamp() const
    {
        return body_->timestamp;
    }

    //----------------------------------------------------------------------------------------------------------
    inline const uint8_t*
    Packet::data2() const
    {
        return reinterpret_cast<const uint8_t*>( body_ ) + body_->offsetLayer2;
    }

    //----------------------------------------------------------------------------------------------------------
    inline unsigned
    Packet::nBytes2() const
    {
        return body_->nBytes2;
    }
}

#endif  // __PACKET_H
