/* packetquery.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2021 Askar Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __PACKETQUERY_H
#define __PACKETQUERY_H 1

#include <libiclass/ipv4>

#include "flowrecord.h"
#include "metadata.h"
#include "packetrecord.h"
#include "pcapwriting.h"
#include "timequery.h"
#include "summaryio.h"

namespace trafficmeter
{
    /**********************************************************************************************************/
    class PacketQuery : public TimeQuery
    {
    public:
        struct Stat;

                                        PacketQuery();

        void                            fetch( PcapWriting& pcap, Stat& statistics );
        void                            setIp( iclass::ipaddr4t );
        iclass::ipaddr4t                getIp() const;

    protected:
        iclass::ipaddr4t                ip_;
        int                             hash_;
    };

    /**********************************************************************************************************/
    class PacketQuery::Stat
    {
    public:
        void                            add( const Metadata& meta, bool sHit, bool dHit );
        void                            countFlows();
        bool                            shouldCountFlows() const;
        SummaryIo                       getSummary() const;

    protected:
        std::set<BeFlow>                flowsIn_;
        std::set<BeFlow>                flowsOut_;
        uintmax_t                       nPacketsIn_ = 0;
        uintmax_t                       nPacketsOut_ = 0;
        uintmax_t                       nBytesIn_ = 0;
        uintmax_t                       nBytesOut_ = 0;
        bool                            shouldCountFlows_ = false;
    };
}

#endif  // __PACKETQUERY_H
