/* path.cc
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2021 Askar Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include <algorithm>
#include <cassert>
#include <cerrno>
#include <cstring>
#include <stdexcept>
#include <unistd.h>
#include <sys/stat.h>

#include "path.h"
#include "tmdefs.h"

using namespace trafficmeter;

using std::max;
using std::runtime_error;
using std::string;
using std::to_string;
using std::literals::string_literals::operator""s;

const Path  Path::base( STATEINFO_DIR + "/"s + PACKAGE );

/**************************************************************************************************************/
Path::Path( const string& path ) :
    string( path )
{
}

/**************************************************************************************************************/
Path::Path( string&& path ) :
    string( path )
{
}

/**************************************************************************************************************/
Path::Path( const char* path ) :
    string( path )
{
}

/**************************************************************************************************************/
Path&
Path::operator=( const string& path )
{
    assign( path );
    return *this;
}

/**************************************************************************************************************/
Path&
Path::operator=( string&& path )
{
    assign( path );
    return *this;
}

/**************************************************************************************************************/
Path&
Path::operator=( const char* path )
{
    assign( path );
    return *this;
}

/**************************************************************************************************************/
Path
Path::subdir( int hash, int hashLength ) const
{
    if ( hashLength == 0 )
    {
        return *this;
    }

    char    hashString[] = "00000000";

    for ( char* p = hashString + 7; hash; --p, hash >>= 4 )
    {
        *p = "0123456789abcdef"[ hash & 0xf ];
    }

    return move( *this + '/' + (hashString + 7 - ((hashLength-1)>>2)) );
}

/**************************************************************************************************************/
bool
Path::isExists() const
{
    return access( c_str(), F_OK ) == 0;
}

/**************************************************************************************************************/
string
Path::fileName() const
{
    auto    lastSlash = find_last_of( '/' );

    return ( lastSlash == string::npos )? data() : data() + lastSlash + 1;
}

/**************************************************************************************************************/
uintmax_t
Path::fileSize() const
{
    struct stat     fst;
    int             rcStat = stat( c_str(), &fst );

    if ( rcStat == -1 )
    {
        throw runtime_error( *this + " - " + strerror(errno) );
    }

    return fst.st_size;
}

/**************************************************************************************************************/
time_t
Path::mtime() const
{
    struct stat     fst;
    int             rcStat = stat( c_str(), &fst );

    if ( rcStat == -1 )
    {
        throw runtime_error( *this + " - " + strerror(errno) );
    }

    return fst.st_mtime;
}
