/* stat2query.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __STAT2QUERY_H
#define __STAT2QUERY_H 1

#include <map>
#include <set>

#include "checkcast.h"
#include "groupbytime.h"
#include "timequery.h"
#include "summaryio.h"

namespace trafficmeter
{
    /**********************************************************************************************************/
    class Stat2Query : public TimeQuery
    {
    public:
        struct Key;

                                        Stat2Query();
                                        ~Stat2Query();

        bool                            timeByHour();
        bool                            timeByDay();
        bool                            timeByMonth();
        bool                            hideTime();
        void                            saveVlans();
        void                            saveHosts();
        bool                            setUnicastFilter();
        bool                            setBroadcastFilter();
        void                            invert();
        void                            level3();
        void                            addMac( const MacAddress& );
        void                            fetch( std::map<Key,SummaryIo>& );
        time_t                          regroup( time_t ) const;
        bool                            shouldSaveVlans() const;
        bool                            shouldSaveHosts() const;
        bool                            checkCast( MacAddress ) const;
        bool                            isInverted() const;
        bool                            isLevel3() const;
        const std::set<MacAddress>&     macs() const;

    protected:
        struct Search_;

        GroupByTime*                    groupByTime_;
        bool                            shouldSaveVlans_;
        bool                            shouldSaveHosts_;
        CheckCast*                      checkCast_;
        bool                            isInverted_;
        bool                            isLevel3_;
        std::set<MacAddress>            macs_;
    };

    /**********************************************************************************************************/
    struct Stat2Query::Key
    {
        time_t                          group;
        MacAddress                      mac;
        int                             vlan;

        bool                            operator<( const Key& ) const;
    };
}

/*
****************************************************************************************************************
****************************************************************************************************************
*/

namespace trafficmeter
{
    //----------------------------------------------------------------------------------------------------------
    inline time_t
    Stat2Query::regroup( time_t group ) const
    {
        return groupByTime_? (*groupByTime_)( group ) : group;
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    Stat2Query::shouldSaveVlans() const
    {
        return shouldSaveVlans_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    Stat2Query::shouldSaveHosts() const
    {
        return shouldSaveHosts_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    Stat2Query::checkCast( MacAddress mac ) const
    {
        return checkCast_? (*checkCast_)( mac ) : true;
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    Stat2Query::isInverted() const
    {
        return isInverted_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    Stat2Query::isLevel3() const
    {
        return isLevel3_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline const std::set<MacAddress>&
    Stat2Query::macs() const
    {
        return macs_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline bool
    Stat2Query::Key::operator<( const Key& rhs ) const
    {
        if ( group != rhs.group )
        {
            return group < rhs.group;
        }

        if ( mac != rhs.mac )
        {
            return mac < rhs.mac;
        }

        return vlan < rhs.vlan;
    }
}

#endif  // __STAT2QUERY_H
