/* statcollection.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __STATCOLLECTION_H
#define __STATCOLLECTION_H 1

#include <ctime>
#include <map>
#include <vector>
#include <libiclass/common>

#include "flowrecord.h"
#include "statrecord.h"
#include "tmdefs.h"

namespace trafficmeter
{
    /**********************************************************************************************************/
    class StatCollection : public iclass::commont
    {
    public:
        using SlotVector                = std::vector<std::map<BeStat,SummaryIo>>;

                                        StatCollection( int threadNo, int nThreads );
                                        ~StatCollection();

        void                            sPush( int slotNo, const BeFlow& beFlow, const Summary& summary );
        void                            dPush( int slotNo, const BeFlow& beFlow, const Summary& summary );
        void                            finish( time_t group );
        SlotVector*                     pull();
        time_t                          group() const;
        bool                            isPullable() const;
        long                            nRows() const;

    protected:
        const int                       threadNo_;
        const int                       nSlots_;
        SlotVector*                     slots_c_;
        SlotVector*                     pullSlots_c_;
        SlotVector*                     nextPullSlots_c_;
        time_t                          group_c_;
    };
}

#endif  // __STATCOLLECTION_H
