/* summary.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __SUMMARY_H
#define __SUMMARY_H 1

#include <cstdint>

namespace trafficmeter
{
    /**********************************************************************************************************/
    class Summary
    {
    public:
                                        Summary();
                                        Summary( uintmax_t nPackets, uintmax_t nBytes );

        Summary&                        operator+=( const Summary& );
        uintmax_t                       nPackets() const;
        uintmax_t                       nBytes() const;

    protected:
        uintmax_t                       nPackets_;
        uintmax_t                       nBytes_;
    };
}

/*
****************************************************************************************************************
****************************************************************************************************************
*/

namespace trafficmeter
{
    //----------------------------------------------------------------------------------------------------------
    inline
    Summary::Summary() :
        nPackets_( 0 ),
        nBytes_( 0 )
    {
    }

    //----------------------------------------------------------------------------------------------------------
    inline
    Summary::Summary( uintmax_t nPackets, uintmax_t nBytes ) :
        nPackets_( nPackets ),
        nBytes_( nBytes )
    {
    }

    //----------------------------------------------------------------------------------------------------------
    inline Summary&
    Summary::operator+=( const Summary& s )
    {
        nPackets_ += s.nPackets_;
        nBytes_ += s.nBytes_;
        return *this;
    }

    //----------------------------------------------------------------------------------------------------------
    inline uintmax_t
    Summary::nPackets() const
    {
        return nPackets_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline uintmax_t
    Summary::nBytes() const
    {
        return nBytes_;
    }
}

#endif  // __SUMMARY_H
