/* timequery.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __TIMEQUERY_H
#define __TIMEQUERY_H 1

#include <set>
#include <vector>
#include <libiclass/period>

#include "fileid.h"

namespace trafficmeter
{
    /**********************************************************************************************************/
    class TimeQuery : public iclass::periodt
    {
    public:
        struct File;
        using Range                     = std::pair<std::vector<File>::iterator, std::vector<File>::iterator>;

                                        TimeQuery( const std::string& dirName, int hashLength );
        virtual                         ~TimeQuery();

        void                            readDirectory( const std::set<int>& hashes );
        Range                           getFiles( time_t timeBound );
        bool                            arrangeInterval();
        bool                            setVlanFilter( int vlan );
        bool                            checkVlan( int vlan ) const;
        Path                            filePath( FileId file ) const;

    protected:
        const Path                      dirPath_;
        const int                       hashLength_;
        std::vector<File>               files_;
        int                             vlanFilter_;
    };

    /**********************************************************************************************************/
    struct TimeQuery::File : public FileId
    {
        int                             fd;
        bool                            isScanned;
        uint8_t*                        mapping;
        long                            mapSize;
        long                            offset;
    };
}

/*
****************************************************************************************************************
****************************************************************************************************************
*/

namespace trafficmeter
{
    //----------------------------------------------------------------------------------------------------------
    inline bool
    TimeQuery::checkVlan( int vlan ) const
    {
        return vlanFilter_ < 0 || vlanFilter_ == vlan;
    }
}

#endif  // __TIMEQUERY_H
