/* timestamp.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2020, 2021 Askar
    Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __TIMESTAMP_H
#define __TIMESTAMP_H 1

#include <cstdint>
#include <ctime>

namespace trafficmeter
{
    /**********************************************************************************************************/
    class Timestamp
    {
    public:
        explicit                        Timestamp( uint64_t );
                                        Timestamp( time_t second, long nanoSecond );

                                        operator uint64_t() const;
        time_t                          sec() const;
        long                            usec() const;
        long                            nsec() const;

    protected:
        uint64_t                        timestamp_;
    };
}

/*
****************************************************************************************************************
****************************************************************************************************************
*/

namespace trafficmeter
{
    //----------------------------------------------------------------------------------------------------------
    inline
    Timestamp::Timestamp( uint64_t timestamp ) :
        timestamp_( timestamp )
    {
    }

    //----------------------------------------------------------------------------------------------------------
    inline
    Timestamp::Timestamp( time_t second, long nanoSecond ) :
        timestamp_
        (
            ( static_cast<uint64_t>(second) << 32 ) +
            ( static_cast<uint64_t>(nanoSecond) << 32 ) / 1000000000
        )
    {
    }

    //----------------------------------------------------------------------------------------------------------
    inline
    Timestamp::operator uint64_t() const
    {
        return timestamp_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline time_t
    Timestamp::sec() const
    {
        return timestamp_ >> 32;
    }

    //----------------------------------------------------------------------------------------------------------
    inline long
    Timestamp::usec() const
    {
        return ( (timestamp_ & 0xffffffffULL) * 1000000 ) >> 32;
    }

    //----------------------------------------------------------------------------------------------------------
    inline long
    Timestamp::nsec() const
    {
        return ( (timestamp_ & 0xffffffffULL) * 1000000000 ) >> 32;
    }
}

#endif  // __TIMESTAMP_H
