/* watcher.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2020, 2021 Askar Almatov

    This file is part of Trafficmeter.

    Trafficmeter is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    Trafficmeter is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License along with Trafficmeter. If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef __WATCHER_H
#define __WATCHER_H 1

#include <atomic>
#include <libiclass/task>

#include "resourceusage.h"
#include "source.h"

namespace trafficmeter
{
    /**********************************************************************************************************/
    class Watcher : public iclass::taskt
    {
    public:
                                        Watcher( Source* );
        virtual                         ~Watcher();

        void                            measure();
        uintmax_t                       pps() const;
        uintmax_t                       ppsMin() const;
        uintmax_t                       ppsMax() const;
        uintmax_t                       ppsAverage() const;
        uintmax_t                       bps() const;
        uintmax_t                       bpsMin() const;
        uintmax_t                       bpsMax() const;
        uintmax_t                       bpsAverage() const;
        double                          cpuUsage() const;

    protected:
        virtual void                    routine() override;
        void                            waitForSecond_();

        Source*                         source_;
        const int                       phyAddendum_;
        ResourceUsage                   usage_;
        uintmax_t                       pps_;
        uintmax_t                       ppsMin_;
        uintmax_t                       ppsMax_;
        uintmax_t                       ppsSum_;
        uintmax_t                       bps_;
        uintmax_t                       bpsMin_;
        uintmax_t                       bpsMax_;
        uintmax_t                       bpsSum_;
        uintmax_t                       nPackets2Previous_;
        uintmax_t                       nBytes2Previous_;
        long                            nMeasurings_;
        long                            startUsec_;
        long                            previousUsec_;
        long                            measuringDuration_;
        std::atomic<uintmax_t>          nPackets2_a_;
        std::atomic<uintmax_t>          nBytes2_a_;
    };
}

/*
****************************************************************************************************************
****************************************************************************************************************
*/

namespace trafficmeter
{
    //----------------------------------------------------------------------------------------------------------
    inline uintmax_t
    Watcher::pps() const
    {
        return pps_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline uintmax_t
    Watcher::ppsMin() const
    {
        return ppsMin_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline uintmax_t
    Watcher::ppsMax() const
    {
        return ppsMax_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline uintmax_t
    Watcher::ppsAverage() const
    {
        return ppsSum_ / nMeasurings_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline uintmax_t
    Watcher::bps() const
    {
        return bps_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline uintmax_t
    Watcher::bpsMin() const
    {
        return bpsMin_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline uintmax_t
    Watcher::bpsMax() const
    {
        return bpsMax_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline uintmax_t
    Watcher::bpsAverage() const
    {
        return bpsSum_ / nMeasurings_;
    }

    //----------------------------------------------------------------------------------------------------------
    inline double
    Watcher::cpuUsage() const
    {
        return usage_.seconds();
    }
}

#endif  // __WATCHER_H
